var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

// classes
import CrioArray from './CrioArray';
import CrioObject from './CrioObject';

// is
import { isArray, isCrio, isObject, isReactElement } from './is';

/**
 * @function createIterator
 *
 * @description
 * create a new iterator method
 *
 * @returns {function} the iterator method
 */
export var createIterator = function createIterator() {
  return function iterator() {
    var _this = this;

    var keys = this.keys();
    var length = keys.length;

    var index = 0,
        value = void 0;

    return {
      next: function next() {
        if (index < length) {
          value = _this[keys[index++]];

          return {
            done: false,
            value: value
          };
        }

        return {
          done: true
        };
      }
    };
  };
};

/**
 * @function every
 *
 * @description
 * does every result from calling fn match
 *
 * @param {CrioArray|CrioObject} object the object to test
 * @param {function} fn the function to perform the test on each item with
 * @returns {boolean} does every item match
 */
export var every = function every(object, fn) {
  var objectKeys = object.keys();

  var key = void 0;

  for (var index = 0; index < objectKeys.length; index++) {
    key = objectKeys[index];

    if (!fn(object[key], key, object)) {
      return false;
    }
  }

  return true;
};

/**
 * @function find
 *
 * @description
 * find an item in the array if it exists
 *
 * @param {CrioArray|CrioObject} object the object to search
 * @param {function} fn function to test for finding the item
 * @param {boolean} [isKey] is the search for a key
 * @param {boolean} [isFromEnd] is the search for a key
 * @returns {*} found item or undefined
 */
export var find = function find(object, fn, isKey, isFromEnd) {
  var objectKeys = isFromEnd ? object.keys().reverse() : object.keys();

  var key = void 0;

  for (var index = 0; index < objectKeys.length; index++) {
    key = objectKeys[index];

    if (fn(object[key], key, object)) {
      return isKey ? key : object[key];
    }
  }

  if (isKey) {
    return object.isArray() ? -1 : undefined;
  }

  return undefined;
};

/**
 * @function getCrioedObject
 *
 * @description
 * get the object converted to a CrioArray or CrioObject, if applicable
 *
 * @param {*} object the object to potentially crio
 * @returns {*} either the crioed object, or the object itself
 */
export var getCrioedObject = function getCrioedObject(object) {
  if (!object || (typeof object === 'undefined' ? 'undefined' : _typeof(object)) !== 'object') {
    return object;
  }

  if (isArray(object)) {
    return object instanceof CrioArray ? object : new CrioArray(object);
  }

  return isObject(object) ? new CrioObject(object) : object;
};

/**
 * @function getEntries
 *
 * @description
 * get the [key, value] pairs of the object
 *
 * @param {CrioArray|CrioObject} object the object to get the entries of
 * @returns {CrioArray} the entries of the object
 */
export var getEntries = function getEntries(object) {
  return object.keys().map(function (key) {
    return [key, object[key]];
  });
};

/**
 * @function getRelativeValue
 *
 * @description
 * get the relative value used in copyWithin
 *
 * @param {number} value value used as baseline
 * @param {number} length the length of the crio
 * @returns {number} the relative number value
 */
export var getRelativeValue = function getRelativeValue(value, length) {
  return value < 0 ? Math.max(length + value, 0) : Math.min(value, length);
};

/**
 * @function getValues
 *
 * @description
 * get the values of the object
 *
 * @param {CrioArray|CrioObject} object the object to get the values of
 * @returns {CrioArray} the values of the object
 */
export var getValues = function getValues(object) {
  return object.keys().map(function (key) {
    return object[key];
  });
};

/**
 * @function some
 *
 * @description
 * does any result from calling fn match
 *
 * @param {CrioArray|CrioObject} object the object to test
 * @param {function} fn the function to perform the test on each item with
 * @returns {boolean} does any item match
 */
export var some = function some(object, fn) {
  var objectKeys = object.keys();

  var key = void 0;

  for (var index = 0; index < objectKeys.length; index++) {
    key = objectKeys[index];

    if (fn(object[key], key, object)) {
      return true;
    }
  }

  return false;
};

/**
 * @function thaw
 *
 * @description
 * convert the CrioArray or CrioObject passed to a plain JS object
 *
 * @param {CrioArray|CrioObject} object the object to convert
 * @returns {Array|Object} the plain JS version of the object passed
 */
export var thaw = function thaw(object) {
  if (!isCrio(object)) {
    return object;
  }

  return object.isArray() ? [].concat(object).map(function (item) {
    return thaw(item);
  }) : Object.keys(object).reduce(function (reducedObject, key) {
    reducedObject[key] = thaw(object[key]);

    return reducedObject;
  }, {});
};