'use strict';

exports.__esModule = true;
exports.thaw = exports.some = exports.getValues = exports.getRelativeValue = exports.getEntries = exports.getCrioedObject = exports.find = exports.every = exports.createIterator = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; // classes


// is


var _CrioArray = require('./CrioArray');

var _CrioArray2 = _interopRequireDefault(_CrioArray);

var _CrioObject = require('./CrioObject');

var _CrioObject2 = _interopRequireDefault(_CrioObject);

var _is = require('./is');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @function createIterator
 *
 * @description
 * create a new iterator method
 *
 * @returns {function} the iterator method
 */
var createIterator = exports.createIterator = function createIterator() {
  return function iterator() {
    var _this = this;

    var keys = this.keys();
    var length = keys.length;

    var index = 0,
        value = void 0;

    return {
      next: function next() {
        if (index < length) {
          value = _this[keys[index++]];

          return {
            done: false,
            value: value
          };
        }

        return {
          done: true
        };
      }
    };
  };
};

/**
 * @function every
 *
 * @description
 * does every result from calling fn match
 *
 * @param {CrioArray|CrioObject} object the object to test
 * @param {function} fn the function to perform the test on each item with
 * @returns {boolean} does every item match
 */
var every = exports.every = function every(object, fn) {
  var objectKeys = object.keys();

  var key = void 0;

  for (var index = 0; index < objectKeys.length; index++) {
    key = objectKeys[index];

    if (!fn(object[key], key, object)) {
      return false;
    }
  }

  return true;
};

/**
 * @function find
 *
 * @description
 * find an item in the array if it exists
 *
 * @param {CrioArray|CrioObject} object the object to search
 * @param {function} fn function to test for finding the item
 * @param {boolean} [isKey] is the search for a key
 * @param {boolean} [isFromEnd] is the search for a key
 * @returns {*} found item or undefined
 */
var find = exports.find = function find(object, fn, isKey, isFromEnd) {
  var objectKeys = isFromEnd ? object.keys().reverse() : object.keys();

  var key = void 0;

  for (var index = 0; index < objectKeys.length; index++) {
    key = objectKeys[index];

    if (fn(object[key], key, object)) {
      return isKey ? key : object[key];
    }
  }

  if (isKey) {
    return object.isArray() ? -1 : undefined;
  }

  return undefined;
};

/**
 * @function getCrioedObject
 *
 * @description
 * get the object converted to a CrioArray or CrioObject, if applicable
 *
 * @param {*} object the object to potentially crio
 * @returns {*} either the crioed object, or the object itself
 */
var getCrioedObject = exports.getCrioedObject = function getCrioedObject(object) {
  if (!object || (typeof object === 'undefined' ? 'undefined' : _typeof(object)) !== 'object') {
    return object;
  }

  if ((0, _is.isArray)(object)) {
    return object instanceof _CrioArray2.default ? object : new _CrioArray2.default(object);
  }

  return (0, _is.isObject)(object) ? new _CrioObject2.default(object) : object;
};

/**
 * @function getEntries
 *
 * @description
 * get the [key, value] pairs of the object
 *
 * @param {CrioArray|CrioObject} object the object to get the entries of
 * @returns {CrioArray} the entries of the object
 */
var getEntries = exports.getEntries = function getEntries(object) {
  return object.keys().map(function (key) {
    return [key, object[key]];
  });
};

/**
 * @function getRelativeValue
 *
 * @description
 * get the relative value used in copyWithin
 *
 * @param {number} value value used as baseline
 * @param {number} length the length of the crio
 * @returns {number} the relative number value
 */
var getRelativeValue = exports.getRelativeValue = function getRelativeValue(value, length) {
  return value < 0 ? Math.max(length + value, 0) : Math.min(value, length);
};

/**
 * @function getValues
 *
 * @description
 * get the values of the object
 *
 * @param {CrioArray|CrioObject} object the object to get the values of
 * @returns {CrioArray} the values of the object
 */
var getValues = exports.getValues = function getValues(object) {
  return object.keys().map(function (key) {
    return object[key];
  });
};

/**
 * @function some
 *
 * @description
 * does any result from calling fn match
 *
 * @param {CrioArray|CrioObject} object the object to test
 * @param {function} fn the function to perform the test on each item with
 * @returns {boolean} does any item match
 */
var some = exports.some = function some(object, fn) {
  var objectKeys = object.keys();

  var key = void 0;

  for (var index = 0; index < objectKeys.length; index++) {
    key = objectKeys[index];

    if (fn(object[key], key, object)) {
      return true;
    }
  }

  return false;
};

/**
 * @function thaw
 *
 * @description
 * convert the CrioArray or CrioObject passed to a plain JS object
 *
 * @param {CrioArray|CrioObject} object the object to convert
 * @returns {Array|Object} the plain JS version of the object passed
 */
var thaw = exports.thaw = function thaw(object) {
  if (!(0, _is.isCrio)(object)) {
    return object;
  }

  return object.isArray() ? [].concat(object).map(function (item) {
    return thaw(item);
  }) : Object.keys(object).reduce(function (reducedObject, key) {
    reducedObject[key] = thaw(object[key]);

    return reducedObject;
  }, {});
};